#ifndef _PCE_KMU_H_
#define _PCE_KMU_H_

#include <stdint.h>
#include <stdbool.h>
#include "config.h"

typedef void * PCE_KMU_HANDLE;

/**
 *@ Description: modify prekey, the key should be encrypt by sm2 with a pubkey from kmu.
 *@ session: [in] session handle
 *@ cipher: [in] ciphered key
 *@ csize: [in] ciphered key len
 *@ authen: [in] 
 *@ authensize: [in] 
 *@ return 0: success others: error
 */
int pce_kmu_deploy(PCE_KMU_HANDLE session, uint8_t *cipher, uint32_t csize, uint8_t *authen, uint32_t authensize);

/**
 *@ Description: Open a session for kmu.
 *@ session: [out] buffer to save session handle
 *@ return 0: success others: error
 */
int pce_kmu_open_session(PCE_KMU_HANDLE *session, bool administrator, uint8_t rand1[32], uint8_t rand2[32]);

/**
 *@ Description: Free  session for kmu.
 *@ session: [in] session handle
 *@ return 0: success others: error
 */
int pce_kmu_close_session(PCE_KMU_HANDLE session);


/**
 *@ Description: Get sm2 pubkey from TEE, use the pubkey to encrypt key before inject key.
 *               KMU server will decrypt the ciphered key and use it later.
 *@ session: [in] session handle
 *@ pub_key: [out] buffer to save pubkey
 *@ key_size: [in, out] size of pubkey, return the size of sm2 pubkey
 *@ return 0: success others: error
 */
int pce_kmu_get_pubkey(PCE_KMU_HANDLE session, uint8_t *pub_key, size_t *key_size);

/**
 *@ Description: Inject a user key to kmu, the key should be encrypt by sm2 with a pubkey from kmu.
 *@ session: [in] session handle
 *@ cipher: [in] ciphered key
 *@ cipher_len: [in] ciphered key len
 *@ root_auth_code: [in] 
 *@ pwd_auth_code: [in] 
 *@ key_index: [out] key index from kmu
 *@ return 0: success others: error
 */
int pce_kmu_inject_key(PCE_KMU_HANDLE session, uint8_t *ciphered_key, uint32_t ciphered_key_len, 
                        uint8_t *root_auth_code, uint32_t root_auth_len, 
                        uint32_t keylen, uint32_t *key_index, uint32_t key_type, uint32_t key_off, uint32_t psw_len);

/**
 *@ Description: Uninject a user key from kmu.
 *@ session: [in] session handle
 *@ key_index: [in] ciphered key
 *@ root_auth_code: [in] ciphered key len
 *@ pwd_auth_code: [in] 
 *@ return 0: success others: error
 */
int pce_kmu_uninject_key(PCE_KMU_HANDLE session, uint32_t key_index, 
                        uint8_t *root_auth_code, uint32_t root_auth_len);

/**
 *@ Description: Enabled a key in kmu.
 *@ session: [in] session handle
 *@ key_index: [in] ciphered key
 *@ pwd_auth_code: [in] auth code of key
 *@ numa: [in] 
 *@ keyid: [out] slot id in specified numa
 *@ return 0: success others: error
 */
int pce_kmu_enable_key(PCE_KMU_HANDLE session, uint32_t key_index, 
                        uint8_t *passphrase, uint32_t passphrase_len,
                        uint8_t numa, uint32_t *key_id, uint8_t challenge[32]);

/**
 *@ Description: Disabled a key in kmu.
 *@ session: [in] session handle
 *@ key_index: [in] ciphered key
 *@ pwd_auth_code: [in] auth code of key
 *@ numa: [in] 
 *@ keyid: [in] slot id in specified numa
 *@ return 0: success others: error
 */
int pce_kmu_disable_key(PCE_KMU_HANDLE session, uint32_t key_index, 
                        uint8_t *passphrase, uint32_t passphrase_len,
                        uint8_t numa, uint32_t key_id, uint8_t challenge[32]);

/**
 *@ Description: Get kmu feature status, to check if compiled with kmu feature.
 *
 * if make with KMU=y, it will return 1, else return 0
 *
 *@ return 0: disabled others: enabled
 */
int pce_kmu_feature_enabled(void);

#endif

